#include <linux/types.h>
#include <linux/errno.h>
#include <verifier/common.h>
#include <linux/spinlock.h>

struct mutex;

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "ps_lock_of_ath10k_pci" was not locked and lock it */
void ldv_mutex_lock_ps_lock_of_ath10k_pci(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "ps_lock_of_ath10k_pci" */
        ldv_mutex_model_lock(lock,  "ps_lock_of_ath10k_pci" );
}


/* MODEL_FUNC_DEF Check that mutex "ps_lock_of_ath10k_pci" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_ps_lock_of_ath10k_pci(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_ps_lock_of_ath10k_pci(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "ps_lock_of_ath10k_pci" was locked in this or another thread  */
int ldv_mutex_is_acquired_ps_lock_of_ath10k_pci(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "ps_lock_of_ath10k_pci");
}

/* MODEL_FUNC_DEF Lock mutex "ps_lock_of_ath10k_pci" if it was not locked before */
int ldv_mutex_trylock_ps_lock_of_ath10k_pci(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_ps_lock_of_ath10k_pci(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "ps_lock_of_ath10k_pci" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_ps_lock_of_ath10k_pci(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_ps_lock_of_ath10k_pci(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "ps_lock_of_ath10k_pci" was locked and unlock it */
void ldv_mutex_unlock_ps_lock_of_ath10k_pci(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "ps_lock_of_ath10k_pci");
}

/* MODEL_FUNC_DEF Check that spinlock "ps_lock_of_ath10k_pci" was not locked and lock it */
void ldv_spin_lock_ps_lock_of_ath10k_pci(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "ps_lock_of_ath10k_pci");
}

/* MODEL_FUNC_DEF Check that spinlock "ps_lock_of_ath10k_pci" was locked and unlock it */
void ldv_spin_unlock_ps_lock_of_ath10k_pci(void)
{
	/* CHANGE_STATE Unlock spinlock "ps_lock_of_ath10k_pci" */
	ldv_spin_model_unlock(  "ps_lock_of_ath10k_pci");
}

/* MODEL_FUNC_DEF Check that spinlock "ps_lock_of_ath10k_pci" was not locked and nondeterministically lock it */
int ldv_spin_trylock_ps_lock_of_ath10k_pci(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_ps_lock_of_ath10k_pci();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "ps_lock_of_ath10k_pci" and wait until it will be unlocked */
void ldv_spin_unlock_wait_ps_lock_of_ath10k_pci(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "ps_lock_of_ath10k_pci" was locked */
int ldv_spin_is_locked_ps_lock_of_ath10k_pci(void)
{
	return ldv_spin_model_is_locked(  "ps_lock_of_ath10k_pci");
}

/* MODEL_FUNC_DEF Check whether spinlock "ps_lock_of_ath10k_pci" was not locked */
int ldv_spin_can_lock_ps_lock_of_ath10k_pci(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_ps_lock_of_ath10k_pci();
}

/* MODEL_FUNC_DEF Check whether spinlock "ps_lock_of_ath10k_pci" is contended */
int ldv_spin_is_contended_ps_lock_of_ath10k_pci(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "ps_lock_of_ath10k_pci" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "ps_lock_of_ath10k_pci" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "ps_lock_of_ath10k_pci" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "ps_lock_of_ath10k_pci" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_ps_lock_of_ath10k_pci(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "ps_lock_of_ath10k_pci" */
		ldv_spin_lock_ps_lock_of_ath10k_pci();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "ps_lock_of_ath10k_pci" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "ptl" was not locked and lock it */
void ldv_mutex_lock_ptl(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "ptl" */
        ldv_mutex_model_lock(lock,  "ptl" );
}


/* MODEL_FUNC_DEF Check that mutex "ptl" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_ptl(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_ptl(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "ptl" was locked in this or another thread  */
int ldv_mutex_is_acquired_ptl(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "ptl");
}

/* MODEL_FUNC_DEF Lock mutex "ptl" if it was not locked before */
int ldv_mutex_trylock_ptl(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_ptl(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "ptl" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_ptl(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_ptl(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "ptl" was locked and unlock it */
void ldv_mutex_unlock_ptl(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "ptl");
}

/* MODEL_FUNC_DEF Check that spinlock "ptl" was not locked and lock it */
void ldv_spin_lock_ptl(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "ptl");
}

/* MODEL_FUNC_DEF Check that spinlock "ptl" was locked and unlock it */
void ldv_spin_unlock_ptl(void)
{
	/* CHANGE_STATE Unlock spinlock "ptl" */
	ldv_spin_model_unlock(  "ptl");
}

/* MODEL_FUNC_DEF Check that spinlock "ptl" was not locked and nondeterministically lock it */
int ldv_spin_trylock_ptl(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_ptl();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "ptl" and wait until it will be unlocked */
void ldv_spin_unlock_wait_ptl(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "ptl" was locked */
int ldv_spin_is_locked_ptl(void)
{
	return ldv_spin_model_is_locked(  "ptl");
}

/* MODEL_FUNC_DEF Check whether spinlock "ptl" was not locked */
int ldv_spin_can_lock_ptl(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_ptl();
}

/* MODEL_FUNC_DEF Check whether spinlock "ptl" is contended */
int ldv_spin_is_contended_ptl(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "ptl" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "ptl" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "ptl" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "ptl" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_ptl(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "ptl" */
		ldv_spin_lock_ptl();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "ptl" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "i_lock_of_inode" was not locked and lock it */
void ldv_mutex_lock_i_lock_of_inode(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "i_lock_of_inode" */
        ldv_mutex_model_lock(lock,  "i_lock_of_inode" );
}


/* MODEL_FUNC_DEF Check that mutex "i_lock_of_inode" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_i_lock_of_inode(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_i_lock_of_inode(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "i_lock_of_inode" was locked in this or another thread  */
int ldv_mutex_is_acquired_i_lock_of_inode(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "i_lock_of_inode");
}

/* MODEL_FUNC_DEF Lock mutex "i_lock_of_inode" if it was not locked before */
int ldv_mutex_trylock_i_lock_of_inode(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_i_lock_of_inode(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "i_lock_of_inode" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_i_lock_of_inode(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_i_lock_of_inode(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "i_lock_of_inode" was locked and unlock it */
void ldv_mutex_unlock_i_lock_of_inode(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "i_lock_of_inode");
}

/* MODEL_FUNC_DEF Check that spinlock "i_lock_of_inode" was not locked and lock it */
void ldv_spin_lock_i_lock_of_inode(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "i_lock_of_inode");
}

/* MODEL_FUNC_DEF Check that spinlock "i_lock_of_inode" was locked and unlock it */
void ldv_spin_unlock_i_lock_of_inode(void)
{
	/* CHANGE_STATE Unlock spinlock "i_lock_of_inode" */
	ldv_spin_model_unlock(  "i_lock_of_inode");
}

/* MODEL_FUNC_DEF Check that spinlock "i_lock_of_inode" was not locked and nondeterministically lock it */
int ldv_spin_trylock_i_lock_of_inode(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_i_lock_of_inode();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "i_lock_of_inode" and wait until it will be unlocked */
void ldv_spin_unlock_wait_i_lock_of_inode(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "i_lock_of_inode" was locked */
int ldv_spin_is_locked_i_lock_of_inode(void)
{
	return ldv_spin_model_is_locked(  "i_lock_of_inode");
}

/* MODEL_FUNC_DEF Check whether spinlock "i_lock_of_inode" was not locked */
int ldv_spin_can_lock_i_lock_of_inode(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_i_lock_of_inode();
}

/* MODEL_FUNC_DEF Check whether spinlock "i_lock_of_inode" is contended */
int ldv_spin_is_contended_i_lock_of_inode(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "i_lock_of_inode" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "i_lock_of_inode" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "i_lock_of_inode" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "i_lock_of_inode" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_i_lock_of_inode(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "i_lock_of_inode" */
		ldv_spin_lock_i_lock_of_inode();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "i_lock_of_inode" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "_xmit_lock_of_netdev_queue" was not locked and lock it */
void ldv_mutex_lock__xmit_lock_of_netdev_queue(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "_xmit_lock_of_netdev_queue" */
        ldv_mutex_model_lock(lock,  "_xmit_lock_of_netdev_queue" );
}


/* MODEL_FUNC_DEF Check that mutex "_xmit_lock_of_netdev_queue" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable__xmit_lock_of_netdev_queue(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock__xmit_lock_of_netdev_queue(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "_xmit_lock_of_netdev_queue" was locked in this or another thread  */
int ldv_mutex_is_acquired__xmit_lock_of_netdev_queue(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "_xmit_lock_of_netdev_queue");
}

/* MODEL_FUNC_DEF Lock mutex "_xmit_lock_of_netdev_queue" if it was not locked before */
int ldv_mutex_trylock__xmit_lock_of_netdev_queue(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock__xmit_lock_of_netdev_queue(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "_xmit_lock_of_netdev_queue" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock__xmit_lock_of_netdev_queue(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock__xmit_lock_of_netdev_queue(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "_xmit_lock_of_netdev_queue" was locked and unlock it */
void ldv_mutex_unlock__xmit_lock_of_netdev_queue(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "_xmit_lock_of_netdev_queue");
}

/* MODEL_FUNC_DEF Check that spinlock "_xmit_lock_of_netdev_queue" was not locked and lock it */
void ldv_spin_lock__xmit_lock_of_netdev_queue(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "_xmit_lock_of_netdev_queue");
}

/* MODEL_FUNC_DEF Check that spinlock "_xmit_lock_of_netdev_queue" was locked and unlock it */
void ldv_spin_unlock__xmit_lock_of_netdev_queue(void)
{
	/* CHANGE_STATE Unlock spinlock "_xmit_lock_of_netdev_queue" */
	ldv_spin_model_unlock(  "_xmit_lock_of_netdev_queue");
}

/* MODEL_FUNC_DEF Check that spinlock "_xmit_lock_of_netdev_queue" was not locked and nondeterministically lock it */
int ldv_spin_trylock__xmit_lock_of_netdev_queue(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock__xmit_lock_of_netdev_queue();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "_xmit_lock_of_netdev_queue" and wait until it will be unlocked */
void ldv_spin_unlock_wait__xmit_lock_of_netdev_queue(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "_xmit_lock_of_netdev_queue" was locked */
int ldv_spin_is_locked__xmit_lock_of_netdev_queue(void)
{
	return ldv_spin_model_is_locked(  "_xmit_lock_of_netdev_queue");
}

/* MODEL_FUNC_DEF Check whether spinlock "_xmit_lock_of_netdev_queue" was not locked */
int ldv_spin_can_lock__xmit_lock_of_netdev_queue(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked__xmit_lock_of_netdev_queue();
}

/* MODEL_FUNC_DEF Check whether spinlock "_xmit_lock_of_netdev_queue" is contended */
int ldv_spin_is_contended__xmit_lock_of_netdev_queue(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "_xmit_lock_of_netdev_queue" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "_xmit_lock_of_netdev_queue" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "_xmit_lock_of_netdev_queue" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "_xmit_lock_of_netdev_queue" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock__xmit_lock_of_netdev_queue(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "_xmit_lock_of_netdev_queue" */
		ldv_spin_lock__xmit_lock_of_netdev_queue();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "_xmit_lock_of_netdev_queue" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "node_size_lock_of_pglist_data" was not locked and lock it */
void ldv_mutex_lock_node_size_lock_of_pglist_data(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "node_size_lock_of_pglist_data" */
        ldv_mutex_model_lock(lock,  "node_size_lock_of_pglist_data" );
}


/* MODEL_FUNC_DEF Check that mutex "node_size_lock_of_pglist_data" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_node_size_lock_of_pglist_data(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_node_size_lock_of_pglist_data(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "node_size_lock_of_pglist_data" was locked in this or another thread  */
int ldv_mutex_is_acquired_node_size_lock_of_pglist_data(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "node_size_lock_of_pglist_data");
}

/* MODEL_FUNC_DEF Lock mutex "node_size_lock_of_pglist_data" if it was not locked before */
int ldv_mutex_trylock_node_size_lock_of_pglist_data(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_node_size_lock_of_pglist_data(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "node_size_lock_of_pglist_data" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_node_size_lock_of_pglist_data(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_node_size_lock_of_pglist_data(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "node_size_lock_of_pglist_data" was locked and unlock it */
void ldv_mutex_unlock_node_size_lock_of_pglist_data(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "node_size_lock_of_pglist_data");
}

/* MODEL_FUNC_DEF Check that spinlock "node_size_lock_of_pglist_data" was not locked and lock it */
void ldv_spin_lock_node_size_lock_of_pglist_data(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "node_size_lock_of_pglist_data");
}

/* MODEL_FUNC_DEF Check that spinlock "node_size_lock_of_pglist_data" was locked and unlock it */
void ldv_spin_unlock_node_size_lock_of_pglist_data(void)
{
	/* CHANGE_STATE Unlock spinlock "node_size_lock_of_pglist_data" */
	ldv_spin_model_unlock(  "node_size_lock_of_pglist_data");
}

/* MODEL_FUNC_DEF Check that spinlock "node_size_lock_of_pglist_data" was not locked and nondeterministically lock it */
int ldv_spin_trylock_node_size_lock_of_pglist_data(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_node_size_lock_of_pglist_data();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "node_size_lock_of_pglist_data" and wait until it will be unlocked */
void ldv_spin_unlock_wait_node_size_lock_of_pglist_data(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "node_size_lock_of_pglist_data" was locked */
int ldv_spin_is_locked_node_size_lock_of_pglist_data(void)
{
	return ldv_spin_model_is_locked(  "node_size_lock_of_pglist_data");
}

/* MODEL_FUNC_DEF Check whether spinlock "node_size_lock_of_pglist_data" was not locked */
int ldv_spin_can_lock_node_size_lock_of_pglist_data(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_node_size_lock_of_pglist_data();
}

/* MODEL_FUNC_DEF Check whether spinlock "node_size_lock_of_pglist_data" is contended */
int ldv_spin_is_contended_node_size_lock_of_pglist_data(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "node_size_lock_of_pglist_data" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "node_size_lock_of_pglist_data" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "node_size_lock_of_pglist_data" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "node_size_lock_of_pglist_data" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_node_size_lock_of_pglist_data(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "node_size_lock_of_pglist_data" */
		ldv_spin_lock_node_size_lock_of_pglist_data();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "node_size_lock_of_pglist_data" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "addr_list_lock_of_net_device" was not locked and lock it */
void ldv_mutex_lock_addr_list_lock_of_net_device(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "addr_list_lock_of_net_device" */
        ldv_mutex_model_lock(lock,  "addr_list_lock_of_net_device" );
}


/* MODEL_FUNC_DEF Check that mutex "addr_list_lock_of_net_device" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_addr_list_lock_of_net_device(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_addr_list_lock_of_net_device(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "addr_list_lock_of_net_device" was locked in this or another thread  */
int ldv_mutex_is_acquired_addr_list_lock_of_net_device(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "addr_list_lock_of_net_device");
}

/* MODEL_FUNC_DEF Lock mutex "addr_list_lock_of_net_device" if it was not locked before */
int ldv_mutex_trylock_addr_list_lock_of_net_device(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_addr_list_lock_of_net_device(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "addr_list_lock_of_net_device" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_addr_list_lock_of_net_device(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_addr_list_lock_of_net_device(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "addr_list_lock_of_net_device" was locked and unlock it */
void ldv_mutex_unlock_addr_list_lock_of_net_device(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "addr_list_lock_of_net_device");
}

/* MODEL_FUNC_DEF Check that spinlock "addr_list_lock_of_net_device" was not locked and lock it */
void ldv_spin_lock_addr_list_lock_of_net_device(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "addr_list_lock_of_net_device");
}

/* MODEL_FUNC_DEF Check that spinlock "addr_list_lock_of_net_device" was locked and unlock it */
void ldv_spin_unlock_addr_list_lock_of_net_device(void)
{
	/* CHANGE_STATE Unlock spinlock "addr_list_lock_of_net_device" */
	ldv_spin_model_unlock(  "addr_list_lock_of_net_device");
}

/* MODEL_FUNC_DEF Check that spinlock "addr_list_lock_of_net_device" was not locked and nondeterministically lock it */
int ldv_spin_trylock_addr_list_lock_of_net_device(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_addr_list_lock_of_net_device();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "addr_list_lock_of_net_device" and wait until it will be unlocked */
void ldv_spin_unlock_wait_addr_list_lock_of_net_device(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "addr_list_lock_of_net_device" was locked */
int ldv_spin_is_locked_addr_list_lock_of_net_device(void)
{
	return ldv_spin_model_is_locked(  "addr_list_lock_of_net_device");
}

/* MODEL_FUNC_DEF Check whether spinlock "addr_list_lock_of_net_device" was not locked */
int ldv_spin_can_lock_addr_list_lock_of_net_device(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_addr_list_lock_of_net_device();
}

/* MODEL_FUNC_DEF Check whether spinlock "addr_list_lock_of_net_device" is contended */
int ldv_spin_is_contended_addr_list_lock_of_net_device(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "addr_list_lock_of_net_device" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "addr_list_lock_of_net_device" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "addr_list_lock_of_net_device" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "addr_list_lock_of_net_device" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_addr_list_lock_of_net_device(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "addr_list_lock_of_net_device" */
		ldv_spin_lock_addr_list_lock_of_net_device();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "addr_list_lock_of_net_device" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "lock_of_dma_async_tx_descriptor" was not locked and lock it */
void ldv_mutex_lock_lock_of_dma_async_tx_descriptor(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "lock_of_dma_async_tx_descriptor" */
        ldv_mutex_model_lock(lock,  "lock_of_dma_async_tx_descriptor" );
}


/* MODEL_FUNC_DEF Check that mutex "lock_of_dma_async_tx_descriptor" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_lock_of_dma_async_tx_descriptor(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_lock_of_dma_async_tx_descriptor(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "lock_of_dma_async_tx_descriptor" was locked in this or another thread  */
int ldv_mutex_is_acquired_lock_of_dma_async_tx_descriptor(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "lock_of_dma_async_tx_descriptor");
}

/* MODEL_FUNC_DEF Lock mutex "lock_of_dma_async_tx_descriptor" if it was not locked before */
int ldv_mutex_trylock_lock_of_dma_async_tx_descriptor(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_lock_of_dma_async_tx_descriptor(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "lock_of_dma_async_tx_descriptor" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_lock_of_dma_async_tx_descriptor(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_lock_of_dma_async_tx_descriptor(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "lock_of_dma_async_tx_descriptor" was locked and unlock it */
void ldv_mutex_unlock_lock_of_dma_async_tx_descriptor(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "lock_of_dma_async_tx_descriptor");
}

/* MODEL_FUNC_DEF Check that spinlock "lock_of_dma_async_tx_descriptor" was not locked and lock it */
void ldv_spin_lock_lock_of_dma_async_tx_descriptor(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "lock_of_dma_async_tx_descriptor");
}

/* MODEL_FUNC_DEF Check that spinlock "lock_of_dma_async_tx_descriptor" was locked and unlock it */
void ldv_spin_unlock_lock_of_dma_async_tx_descriptor(void)
{
	/* CHANGE_STATE Unlock spinlock "lock_of_dma_async_tx_descriptor" */
	ldv_spin_model_unlock(  "lock_of_dma_async_tx_descriptor");
}

/* MODEL_FUNC_DEF Check that spinlock "lock_of_dma_async_tx_descriptor" was not locked and nondeterministically lock it */
int ldv_spin_trylock_lock_of_dma_async_tx_descriptor(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_lock_of_dma_async_tx_descriptor();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "lock_of_dma_async_tx_descriptor" and wait until it will be unlocked */
void ldv_spin_unlock_wait_lock_of_dma_async_tx_descriptor(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "lock_of_dma_async_tx_descriptor" was locked */
int ldv_spin_is_locked_lock_of_dma_async_tx_descriptor(void)
{
	return ldv_spin_model_is_locked(  "lock_of_dma_async_tx_descriptor");
}

/* MODEL_FUNC_DEF Check whether spinlock "lock_of_dma_async_tx_descriptor" was not locked */
int ldv_spin_can_lock_lock_of_dma_async_tx_descriptor(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_lock_of_dma_async_tx_descriptor();
}

/* MODEL_FUNC_DEF Check whether spinlock "lock_of_dma_async_tx_descriptor" is contended */
int ldv_spin_is_contended_lock_of_dma_async_tx_descriptor(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "lock_of_dma_async_tx_descriptor" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "lock_of_dma_async_tx_descriptor" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "lock_of_dma_async_tx_descriptor" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "lock_of_dma_async_tx_descriptor" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_lock_of_dma_async_tx_descriptor(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "lock_of_dma_async_tx_descriptor" */
		ldv_spin_lock_lock_of_dma_async_tx_descriptor();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "lock_of_dma_async_tx_descriptor" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "siglock_of_sighand_struct" was not locked and lock it */
void ldv_mutex_lock_siglock_of_sighand_struct(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "siglock_of_sighand_struct" */
        ldv_mutex_model_lock(lock,  "siglock_of_sighand_struct" );
}


/* MODEL_FUNC_DEF Check that mutex "siglock_of_sighand_struct" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_siglock_of_sighand_struct(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_siglock_of_sighand_struct(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "siglock_of_sighand_struct" was locked in this or another thread  */
int ldv_mutex_is_acquired_siglock_of_sighand_struct(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "siglock_of_sighand_struct");
}

/* MODEL_FUNC_DEF Lock mutex "siglock_of_sighand_struct" if it was not locked before */
int ldv_mutex_trylock_siglock_of_sighand_struct(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_siglock_of_sighand_struct(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "siglock_of_sighand_struct" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_siglock_of_sighand_struct(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_siglock_of_sighand_struct(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "siglock_of_sighand_struct" was locked and unlock it */
void ldv_mutex_unlock_siglock_of_sighand_struct(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "siglock_of_sighand_struct");
}

/* MODEL_FUNC_DEF Check that spinlock "siglock_of_sighand_struct" was not locked and lock it */
void ldv_spin_lock_siglock_of_sighand_struct(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "siglock_of_sighand_struct");
}

/* MODEL_FUNC_DEF Check that spinlock "siglock_of_sighand_struct" was locked and unlock it */
void ldv_spin_unlock_siglock_of_sighand_struct(void)
{
	/* CHANGE_STATE Unlock spinlock "siglock_of_sighand_struct" */
	ldv_spin_model_unlock(  "siglock_of_sighand_struct");
}

/* MODEL_FUNC_DEF Check that spinlock "siglock_of_sighand_struct" was not locked and nondeterministically lock it */
int ldv_spin_trylock_siglock_of_sighand_struct(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_siglock_of_sighand_struct();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "siglock_of_sighand_struct" and wait until it will be unlocked */
void ldv_spin_unlock_wait_siglock_of_sighand_struct(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "siglock_of_sighand_struct" was locked */
int ldv_spin_is_locked_siglock_of_sighand_struct(void)
{
	return ldv_spin_model_is_locked(  "siglock_of_sighand_struct");
}

/* MODEL_FUNC_DEF Check whether spinlock "siglock_of_sighand_struct" was not locked */
int ldv_spin_can_lock_siglock_of_sighand_struct(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_siglock_of_sighand_struct();
}

/* MODEL_FUNC_DEF Check whether spinlock "siglock_of_sighand_struct" is contended */
int ldv_spin_is_contended_siglock_of_sighand_struct(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "siglock_of_sighand_struct" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "siglock_of_sighand_struct" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "siglock_of_sighand_struct" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "siglock_of_sighand_struct" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_siglock_of_sighand_struct(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "siglock_of_sighand_struct" */
		ldv_spin_lock_siglock_of_sighand_struct();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "siglock_of_sighand_struct" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "lock_of_NOT_ARG_SIGN" was not locked and lock it */
void ldv_mutex_lock_lock_of_NOT_ARG_SIGN(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "lock_of_NOT_ARG_SIGN" */
        ldv_mutex_model_lock(lock,  "lock_of_NOT_ARG_SIGN" );
}


/* MODEL_FUNC_DEF Check that mutex "lock_of_NOT_ARG_SIGN" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_lock_of_NOT_ARG_SIGN(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_lock_of_NOT_ARG_SIGN(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "lock_of_NOT_ARG_SIGN" was locked in this or another thread  */
int ldv_mutex_is_acquired_lock_of_NOT_ARG_SIGN(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "lock_of_NOT_ARG_SIGN");
}

/* MODEL_FUNC_DEF Lock mutex "lock_of_NOT_ARG_SIGN" if it was not locked before */
int ldv_mutex_trylock_lock_of_NOT_ARG_SIGN(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_lock_of_NOT_ARG_SIGN(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "lock_of_NOT_ARG_SIGN" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_lock_of_NOT_ARG_SIGN(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_lock_of_NOT_ARG_SIGN(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "lock_of_NOT_ARG_SIGN" was locked and unlock it */
void ldv_mutex_unlock_lock_of_NOT_ARG_SIGN(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "lock_of_NOT_ARG_SIGN");
}

/* MODEL_FUNC_DEF Check that spinlock "lock_of_NOT_ARG_SIGN" was not locked and lock it */
void ldv_spin_lock_lock_of_NOT_ARG_SIGN(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "lock_of_NOT_ARG_SIGN");
}

/* MODEL_FUNC_DEF Check that spinlock "lock_of_NOT_ARG_SIGN" was locked and unlock it */
void ldv_spin_unlock_lock_of_NOT_ARG_SIGN(void)
{
	/* CHANGE_STATE Unlock spinlock "lock_of_NOT_ARG_SIGN" */
	ldv_spin_model_unlock(  "lock_of_NOT_ARG_SIGN");
}

/* MODEL_FUNC_DEF Check that spinlock "lock_of_NOT_ARG_SIGN" was not locked and nondeterministically lock it */
int ldv_spin_trylock_lock_of_NOT_ARG_SIGN(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_lock_of_NOT_ARG_SIGN();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "lock_of_NOT_ARG_SIGN" and wait until it will be unlocked */
void ldv_spin_unlock_wait_lock_of_NOT_ARG_SIGN(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "lock_of_NOT_ARG_SIGN" was locked */
int ldv_spin_is_locked_lock_of_NOT_ARG_SIGN(void)
{
	return ldv_spin_model_is_locked(  "lock_of_NOT_ARG_SIGN");
}

/* MODEL_FUNC_DEF Check whether spinlock "lock_of_NOT_ARG_SIGN" was not locked */
int ldv_spin_can_lock_lock_of_NOT_ARG_SIGN(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_lock_of_NOT_ARG_SIGN();
}

/* MODEL_FUNC_DEF Check whether spinlock "lock_of_NOT_ARG_SIGN" is contended */
int ldv_spin_is_contended_lock_of_NOT_ARG_SIGN(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "lock_of_NOT_ARG_SIGN" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "lock_of_NOT_ARG_SIGN" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "lock_of_NOT_ARG_SIGN" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "lock_of_NOT_ARG_SIGN" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_lock_of_NOT_ARG_SIGN(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "lock_of_NOT_ARG_SIGN" */
		ldv_spin_lock_lock_of_NOT_ARG_SIGN();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "lock_of_NOT_ARG_SIGN" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "data_lock_of_ath10k" was not locked and lock it */
void ldv_mutex_lock_data_lock_of_ath10k(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "data_lock_of_ath10k" */
        ldv_mutex_model_lock(lock,  "data_lock_of_ath10k" );
}


/* MODEL_FUNC_DEF Check that mutex "data_lock_of_ath10k" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_data_lock_of_ath10k(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_data_lock_of_ath10k(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "data_lock_of_ath10k" was locked in this or another thread  */
int ldv_mutex_is_acquired_data_lock_of_ath10k(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "data_lock_of_ath10k");
}

/* MODEL_FUNC_DEF Lock mutex "data_lock_of_ath10k" if it was not locked before */
int ldv_mutex_trylock_data_lock_of_ath10k(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_data_lock_of_ath10k(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "data_lock_of_ath10k" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_data_lock_of_ath10k(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_data_lock_of_ath10k(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "data_lock_of_ath10k" was locked and unlock it */
void ldv_mutex_unlock_data_lock_of_ath10k(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "data_lock_of_ath10k");
}

/* MODEL_FUNC_DEF Check that spinlock "data_lock_of_ath10k" was not locked and lock it */
void ldv_spin_lock_data_lock_of_ath10k(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "data_lock_of_ath10k");
}

/* MODEL_FUNC_DEF Check that spinlock "data_lock_of_ath10k" was locked and unlock it */
void ldv_spin_unlock_data_lock_of_ath10k(void)
{
	/* CHANGE_STATE Unlock spinlock "data_lock_of_ath10k" */
	ldv_spin_model_unlock(  "data_lock_of_ath10k");
}

/* MODEL_FUNC_DEF Check that spinlock "data_lock_of_ath10k" was not locked and nondeterministically lock it */
int ldv_spin_trylock_data_lock_of_ath10k(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_data_lock_of_ath10k();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "data_lock_of_ath10k" and wait until it will be unlocked */
void ldv_spin_unlock_wait_data_lock_of_ath10k(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "data_lock_of_ath10k" was locked */
int ldv_spin_is_locked_data_lock_of_ath10k(void)
{
	return ldv_spin_model_is_locked(  "data_lock_of_ath10k");
}

/* MODEL_FUNC_DEF Check whether spinlock "data_lock_of_ath10k" was not locked */
int ldv_spin_can_lock_data_lock_of_ath10k(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_data_lock_of_ath10k();
}

/* MODEL_FUNC_DEF Check whether spinlock "data_lock_of_ath10k" is contended */
int ldv_spin_is_contended_data_lock_of_ath10k(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "data_lock_of_ath10k" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "data_lock_of_ath10k" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "data_lock_of_ath10k" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "data_lock_of_ath10k" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_data_lock_of_ath10k(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "data_lock_of_ath10k" */
		ldv_spin_lock_data_lock_of_ath10k();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "data_lock_of_ath10k" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "mutex_of_device" was not locked and lock it */
void ldv_mutex_lock_mutex_of_device(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "mutex_of_device" */
        ldv_mutex_model_lock(lock,  "mutex_of_device" );
}


/* MODEL_FUNC_DEF Check that mutex "mutex_of_device" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_mutex_of_device(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_mutex_of_device(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "mutex_of_device" was locked in this or another thread  */
int ldv_mutex_is_acquired_mutex_of_device(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "mutex_of_device");
}

/* MODEL_FUNC_DEF Lock mutex "mutex_of_device" if it was not locked before */
int ldv_mutex_trylock_mutex_of_device(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_mutex_of_device(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "mutex_of_device" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_mutex_of_device(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_mutex_of_device(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "mutex_of_device" was locked and unlock it */
void ldv_mutex_unlock_mutex_of_device(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "mutex_of_device");
}

/* MODEL_FUNC_DEF Check that spinlock "mutex_of_device" was not locked and lock it */
void ldv_spin_lock_mutex_of_device(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "mutex_of_device");
}

/* MODEL_FUNC_DEF Check that spinlock "mutex_of_device" was locked and unlock it */
void ldv_spin_unlock_mutex_of_device(void)
{
	/* CHANGE_STATE Unlock spinlock "mutex_of_device" */
	ldv_spin_model_unlock(  "mutex_of_device");
}

/* MODEL_FUNC_DEF Check that spinlock "mutex_of_device" was not locked and nondeterministically lock it */
int ldv_spin_trylock_mutex_of_device(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_mutex_of_device();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "mutex_of_device" and wait until it will be unlocked */
void ldv_spin_unlock_wait_mutex_of_device(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "mutex_of_device" was locked */
int ldv_spin_is_locked_mutex_of_device(void)
{
	return ldv_spin_model_is_locked(  "mutex_of_device");
}

/* MODEL_FUNC_DEF Check whether spinlock "mutex_of_device" was not locked */
int ldv_spin_can_lock_mutex_of_device(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_mutex_of_device();
}

/* MODEL_FUNC_DEF Check whether spinlock "mutex_of_device" is contended */
int ldv_spin_is_contended_mutex_of_device(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "mutex_of_device" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "mutex_of_device" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "mutex_of_device" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "mutex_of_device" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_mutex_of_device(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "mutex_of_device" */
		ldv_spin_lock_mutex_of_device();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "mutex_of_device" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "tx_global_lock_of_net_device" was not locked and lock it */
void ldv_mutex_lock_tx_global_lock_of_net_device(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "tx_global_lock_of_net_device" */
        ldv_mutex_model_lock(lock,  "tx_global_lock_of_net_device" );
}


/* MODEL_FUNC_DEF Check that mutex "tx_global_lock_of_net_device" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_tx_global_lock_of_net_device(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_tx_global_lock_of_net_device(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "tx_global_lock_of_net_device" was locked in this or another thread  */
int ldv_mutex_is_acquired_tx_global_lock_of_net_device(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "tx_global_lock_of_net_device");
}

/* MODEL_FUNC_DEF Lock mutex "tx_global_lock_of_net_device" if it was not locked before */
int ldv_mutex_trylock_tx_global_lock_of_net_device(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_tx_global_lock_of_net_device(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "tx_global_lock_of_net_device" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_tx_global_lock_of_net_device(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_tx_global_lock_of_net_device(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "tx_global_lock_of_net_device" was locked and unlock it */
void ldv_mutex_unlock_tx_global_lock_of_net_device(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "tx_global_lock_of_net_device");
}

/* MODEL_FUNC_DEF Check that spinlock "tx_global_lock_of_net_device" was not locked and lock it */
void ldv_spin_lock_tx_global_lock_of_net_device(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "tx_global_lock_of_net_device");
}

/* MODEL_FUNC_DEF Check that spinlock "tx_global_lock_of_net_device" was locked and unlock it */
void ldv_spin_unlock_tx_global_lock_of_net_device(void)
{
	/* CHANGE_STATE Unlock spinlock "tx_global_lock_of_net_device" */
	ldv_spin_model_unlock(  "tx_global_lock_of_net_device");
}

/* MODEL_FUNC_DEF Check that spinlock "tx_global_lock_of_net_device" was not locked and nondeterministically lock it */
int ldv_spin_trylock_tx_global_lock_of_net_device(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_tx_global_lock_of_net_device();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "tx_global_lock_of_net_device" and wait until it will be unlocked */
void ldv_spin_unlock_wait_tx_global_lock_of_net_device(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "tx_global_lock_of_net_device" was locked */
int ldv_spin_is_locked_tx_global_lock_of_net_device(void)
{
	return ldv_spin_model_is_locked(  "tx_global_lock_of_net_device");
}

/* MODEL_FUNC_DEF Check whether spinlock "tx_global_lock_of_net_device" was not locked */
int ldv_spin_can_lock_tx_global_lock_of_net_device(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_tx_global_lock_of_net_device();
}

/* MODEL_FUNC_DEF Check whether spinlock "tx_global_lock_of_net_device" is contended */
int ldv_spin_is_contended_tx_global_lock_of_net_device(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "tx_global_lock_of_net_device" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "tx_global_lock_of_net_device" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "tx_global_lock_of_net_device" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "tx_global_lock_of_net_device" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_tx_global_lock_of_net_device(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "tx_global_lock_of_net_device" */
		ldv_spin_lock_tx_global_lock_of_net_device();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "tx_global_lock_of_net_device" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "lock" was not locked and lock it */
void ldv_mutex_lock_lock(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "lock" */
        ldv_mutex_model_lock(lock,  "lock" );
}


/* MODEL_FUNC_DEF Check that mutex "lock" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_lock(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_lock(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "lock" was locked in this or another thread  */
int ldv_mutex_is_acquired_lock(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "lock");
}

/* MODEL_FUNC_DEF Lock mutex "lock" if it was not locked before */
int ldv_mutex_trylock_lock(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_lock(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "lock" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_lock(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_lock(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "lock" was locked and unlock it */
void ldv_mutex_unlock_lock(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "lock");
}

/* MODEL_FUNC_DEF Check that spinlock "lock" was not locked and lock it */
void ldv_spin_lock_lock(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "lock");
}

/* MODEL_FUNC_DEF Check that spinlock "lock" was locked and unlock it */
void ldv_spin_unlock_lock(void)
{
	/* CHANGE_STATE Unlock spinlock "lock" */
	ldv_spin_model_unlock(  "lock");
}

/* MODEL_FUNC_DEF Check that spinlock "lock" was not locked and nondeterministically lock it */
int ldv_spin_trylock_lock(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_lock();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "lock" and wait until it will be unlocked */
void ldv_spin_unlock_wait_lock(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "lock" was locked */
int ldv_spin_is_locked_lock(void)
{
	return ldv_spin_model_is_locked(  "lock");
}

/* MODEL_FUNC_DEF Check whether spinlock "lock" was not locked */
int ldv_spin_can_lock_lock(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_lock();
}

/* MODEL_FUNC_DEF Check whether spinlock "lock" is contended */
int ldv_spin_is_contended_lock(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "lock" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "lock" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "lock" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "lock" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_lock(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "lock" */
		ldv_spin_lock_lock();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "lock" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "i_mutex_of_inode" was not locked and lock it */
void ldv_mutex_lock_i_mutex_of_inode(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "i_mutex_of_inode" */
        ldv_mutex_model_lock(lock,  "i_mutex_of_inode" );
}


/* MODEL_FUNC_DEF Check that mutex "i_mutex_of_inode" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_i_mutex_of_inode(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_i_mutex_of_inode(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "i_mutex_of_inode" was locked in this or another thread  */
int ldv_mutex_is_acquired_i_mutex_of_inode(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "i_mutex_of_inode");
}

/* MODEL_FUNC_DEF Lock mutex "i_mutex_of_inode" if it was not locked before */
int ldv_mutex_trylock_i_mutex_of_inode(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_i_mutex_of_inode(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "i_mutex_of_inode" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_i_mutex_of_inode(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_i_mutex_of_inode(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "i_mutex_of_inode" was locked and unlock it */
void ldv_mutex_unlock_i_mutex_of_inode(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "i_mutex_of_inode");
}

/* MODEL_FUNC_DEF Check that spinlock "i_mutex_of_inode" was not locked and lock it */
void ldv_spin_lock_i_mutex_of_inode(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "i_mutex_of_inode");
}

/* MODEL_FUNC_DEF Check that spinlock "i_mutex_of_inode" was locked and unlock it */
void ldv_spin_unlock_i_mutex_of_inode(void)
{
	/* CHANGE_STATE Unlock spinlock "i_mutex_of_inode" */
	ldv_spin_model_unlock(  "i_mutex_of_inode");
}

/* MODEL_FUNC_DEF Check that spinlock "i_mutex_of_inode" was not locked and nondeterministically lock it */
int ldv_spin_trylock_i_mutex_of_inode(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_i_mutex_of_inode();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "i_mutex_of_inode" and wait until it will be unlocked */
void ldv_spin_unlock_wait_i_mutex_of_inode(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "i_mutex_of_inode" was locked */
int ldv_spin_is_locked_i_mutex_of_inode(void)
{
	return ldv_spin_model_is_locked(  "i_mutex_of_inode");
}

/* MODEL_FUNC_DEF Check whether spinlock "i_mutex_of_inode" was not locked */
int ldv_spin_can_lock_i_mutex_of_inode(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_i_mutex_of_inode();
}

/* MODEL_FUNC_DEF Check whether spinlock "i_mutex_of_inode" is contended */
int ldv_spin_is_contended_i_mutex_of_inode(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "i_mutex_of_inode" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "i_mutex_of_inode" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "i_mutex_of_inode" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "i_mutex_of_inode" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_i_mutex_of_inode(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "i_mutex_of_inode" */
		ldv_spin_lock_i_mutex_of_inode();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "i_mutex_of_inode" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "ce_lock_of_ath10k_pci" was not locked and lock it */
void ldv_mutex_lock_ce_lock_of_ath10k_pci(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "ce_lock_of_ath10k_pci" */
        ldv_mutex_model_lock(lock,  "ce_lock_of_ath10k_pci" );
}


/* MODEL_FUNC_DEF Check that mutex "ce_lock_of_ath10k_pci" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_ce_lock_of_ath10k_pci(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_ce_lock_of_ath10k_pci(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "ce_lock_of_ath10k_pci" was locked in this or another thread  */
int ldv_mutex_is_acquired_ce_lock_of_ath10k_pci(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "ce_lock_of_ath10k_pci");
}

/* MODEL_FUNC_DEF Lock mutex "ce_lock_of_ath10k_pci" if it was not locked before */
int ldv_mutex_trylock_ce_lock_of_ath10k_pci(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_ce_lock_of_ath10k_pci(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "ce_lock_of_ath10k_pci" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_ce_lock_of_ath10k_pci(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_ce_lock_of_ath10k_pci(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "ce_lock_of_ath10k_pci" was locked and unlock it */
void ldv_mutex_unlock_ce_lock_of_ath10k_pci(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "ce_lock_of_ath10k_pci");
}

/* MODEL_FUNC_DEF Check that spinlock "ce_lock_of_ath10k_pci" was not locked and lock it */
void ldv_spin_lock_ce_lock_of_ath10k_pci(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "ce_lock_of_ath10k_pci");
}

/* MODEL_FUNC_DEF Check that spinlock "ce_lock_of_ath10k_pci" was locked and unlock it */
void ldv_spin_unlock_ce_lock_of_ath10k_pci(void)
{
	/* CHANGE_STATE Unlock spinlock "ce_lock_of_ath10k_pci" */
	ldv_spin_model_unlock(  "ce_lock_of_ath10k_pci");
}

/* MODEL_FUNC_DEF Check that spinlock "ce_lock_of_ath10k_pci" was not locked and nondeterministically lock it */
int ldv_spin_trylock_ce_lock_of_ath10k_pci(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_ce_lock_of_ath10k_pci();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "ce_lock_of_ath10k_pci" and wait until it will be unlocked */
void ldv_spin_unlock_wait_ce_lock_of_ath10k_pci(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "ce_lock_of_ath10k_pci" was locked */
int ldv_spin_is_locked_ce_lock_of_ath10k_pci(void)
{
	return ldv_spin_model_is_locked(  "ce_lock_of_ath10k_pci");
}

/* MODEL_FUNC_DEF Check whether spinlock "ce_lock_of_ath10k_pci" was not locked */
int ldv_spin_can_lock_ce_lock_of_ath10k_pci(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_ce_lock_of_ath10k_pci();
}

/* MODEL_FUNC_DEF Check whether spinlock "ce_lock_of_ath10k_pci" is contended */
int ldv_spin_is_contended_ce_lock_of_ath10k_pci(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "ce_lock_of_ath10k_pci" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "ce_lock_of_ath10k_pci" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "ce_lock_of_ath10k_pci" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "ce_lock_of_ath10k_pci" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_ce_lock_of_ath10k_pci(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "ce_lock_of_ath10k_pci" */
		ldv_spin_lock_ce_lock_of_ath10k_pci();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "ce_lock_of_ath10k_pci" */
	return 0;
}

/* mutex model functions */
extern void ldv_mutex_model_lock(struct mutex *lock, char *sign);
extern int ldv_mutex_model_is_locked(struct mutex *lock, char *sign);
extern void ldv_mutex_model_unlock(struct mutex *lock, char *sign);

/* spin model functions */
extern void ldv_spin_model_lock(char *sign);
extern void ldv_spin_model_unlock(char *sign);
extern int ldv_spin_model_is_locked(char *sign);

/* MODEL_FUNC_DEF Check that mutex "alloc_lock_of_task_struct" was not locked and lock it */
void ldv_mutex_lock_alloc_lock_of_task_struct(struct mutex *lock)
{
        /* CHANGE_STATE Lock mutex "alloc_lock_of_task_struct" */
        ldv_mutex_model_lock(lock,  "alloc_lock_of_task_struct" );
}


/* MODEL_FUNC_DEF Check that mutex "alloc_lock_of_task_struct" was not locked and nondeterministically lock it */
int ldv_mutex_lock_interruptible_or_killable_alloc_lock_of_task_struct(struct mutex *lock)
{
	int nondetermined = ldv_undef_int();

  if(nondetermined)
  {
    /* LDV_COMMENT_RETURN Finish with success */
    ldv_mutex_lock_alloc_lock_of_task_struct(lock);
    return 0;
  }
  else
  {
    /* LDV_COMMENT_RETURN Finish with fail. Mutex is keeped unlocked */
    return -4;
  }
}

/* MODEL_FUNC_DEF Say whether mutex "alloc_lock_of_task_struct" was locked in this or another thread  */
int ldv_mutex_is_acquired_alloc_lock_of_task_struct(struct mutex *lock)
{
	return ldv_mutex_model_is_locked(lock,  "alloc_lock_of_task_struct");
}

/* MODEL_FUNC_DEF Lock mutex "alloc_lock_of_task_struct" if it was not locked before */
int ldv_mutex_trylock_alloc_lock_of_task_struct(struct mutex *lock)
{
  int is_mutex_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock mutex*/
  if (is_mutex_held_by_another_thread)
  {
    /* RETURN Finish with fail */
    return 0;
  }
  else
  {
    /* RETURN Finish with success */
    ldv_mutex_lock_alloc_lock_of_task_struct(lock);
    return 1;
  }
}

/* MODEL_FUNC_DEF Decrease counter by one and if it becomes zero check that mutex "alloc_lock_of_task_struct" was not locked and lock it */
int ldv_atomic_dec_and_mutex_lock_alloc_lock_of_task_struct(atomic_t *cnt, struct mutex *lock)
{
	/* OTHER Assign the result of atomic decrement */
  int atomic_value_after_dec = ldv_undef_int();
  /* ASSERT Check if atomic decrement returns zero */
  if (atomic_value_after_dec == 0)
  {
    /* CHANGE_STATE Lock mutex, as atomic has decremented to zero */
    ldv_mutex_lock_alloc_lock_of_task_struct(lock);
    /* RETURN Return 1 with locked mutex */
    return 1;
  }

  /* RETURN Atomic decrement is still not zero, return 0 without locking mutex */
  return 0;
}

/* MODEL_FUNC_DEF Check that mutex "alloc_lock_of_task_struct" was locked and unlock it */
void ldv_mutex_unlock_alloc_lock_of_task_struct(struct mutex *lock)
{
  ldv_mutex_model_unlock(lock,  "alloc_lock_of_task_struct");
}

/* MODEL_FUNC_DEF Check that spinlock "alloc_lock_of_task_struct" was not locked and lock it */
void ldv_spin_lock_alloc_lock_of_task_struct(void)
{
    /* CHANGE_STATE Lock spin */
    ldv_spin_model_lock(  "alloc_lock_of_task_struct");
}

/* MODEL_FUNC_DEF Check that spinlock "alloc_lock_of_task_struct" was locked and unlock it */
void ldv_spin_unlock_alloc_lock_of_task_struct(void)
{
	/* CHANGE_STATE Unlock spinlock "alloc_lock_of_task_struct" */
	ldv_spin_model_unlock(  "alloc_lock_of_task_struct");
}

/* MODEL_FUNC_DEF Check that spinlock "alloc_lock_of_task_struct" was not locked and nondeterministically lock it */
int ldv_spin_trylock_alloc_lock_of_task_struct(void)
{
	/* OTHER Construct nondetermined result */
	int is_spin_held_by_another_thread = ldv_undef_int();

  /* ASSERT Nondeterministically lock spin */
  if (is_spin_held_by_another_thread)
  {
    /* RETURN Spin was not locked. Finish with fail */
    return 0;
  } else {
    /* CHANGE_STATE Lock spin */
    ldv_spin_lock_alloc_lock_of_task_struct();
    /* RETURN Finish with success */
    return 1;
  }
}

/* MODEL_FUNC_DEF The same thread can not both lock spinlock "alloc_lock_of_task_struct" and wait until it will be unlocked */
void ldv_spin_unlock_wait_alloc_lock_of_task_struct(void)
{
}

/* MODEL_FUNC_DEF Check whether spinlock "alloc_lock_of_task_struct" was locked */
int ldv_spin_is_locked_alloc_lock_of_task_struct(void)
{
	return ldv_spin_model_is_locked(  "alloc_lock_of_task_struct");
}

/* MODEL_FUNC_DEF Check whether spinlock "alloc_lock_of_task_struct" was not locked */
int ldv_spin_can_lock_alloc_lock_of_task_struct(void)
{
	/* RETURN Inverse function for spin_is_locked() */
	return !ldv_spin_model_is_locked_alloc_lock_of_task_struct();
}

/* MODEL_FUNC_DEF Check whether spinlock "alloc_lock_of_task_struct" is contended */
int ldv_spin_is_contended_alloc_lock_of_task_struct(void)
{
	int is_spin_contended;

	/* OTHER Construct nondetermined result */
	is_spin_contended = ldv_undef_int();

	/* OTHER Nondeterministically understand whether spinlock "alloc_lock_of_task_struct" is contended */
	if(is_spin_contended)
	{
		/* RETURN Spinlock "alloc_lock_of_task_struct" is contended */
		return 0;
	}
	else
	{
		/* RETURN Spinlock "alloc_lock_of_task_struct" is not contended */
		return 1;
	}
}

/* MODEL_FUNC_DEF Lock spinlock "alloc_lock_of_task_struct" if atomic decrement result is zero */
int ldv_atomic_dec_and_lock_alloc_lock_of_task_struct(void)
{
	int atomic_value_after_dec;

	/* OTHER Assign result of atomic decrement */
	atomic_value_after_dec = ldv_undef_int();

	/* OTHER Atomic decrement result is zero */
	if (atomic_value_after_dec == 0)
	{
		/* CHANGE_STATE Lock spinlock "alloc_lock_of_task_struct" */
		ldv_spin_lock_alloc_lock_of_task_struct();
		/* RETURN Finish with success */
		return 1;
	}

	/* RETURN Atomic decrement result is not zero. Finish with fail without locking spin "alloc_lock_of_task_struct" */
	return 0;
}

